#include "domain.h"



vector < domain > domain::boundary()
{
  vector < domain > result ;
  for (domain::const_iterator i = (*this).begin(); i!=(*this).end();++i)
    {
      domain _inf = (*this);
      domain _sup = (*this);
      string axe = i->first;
      _inf[i->first] = interval(i->second.inf(),i->second.inf());
      _sup[i->first] = interval(i->second.sup(),i->second.sup());
      result.push_back(_inf);
      result.push_back(_sup);
    }
  return result;
}


vector < domain > domain::visibility(int j)
{
  vector < domain > bords = boundary();
  vector < domain > result;
  domain corner0 = corner(j);
  for (int k = 0; k < bords.size() ;k++)
    {
      if (!corner0.subset(bords[k]))
	result.push_back(bords[k]);
    }
  return result;
}


double domain::width()
{
  double result = 0;
  for (map<string,interval>::const_iterator i = (*this).begin(); i!=(*this).end(); ++i)
    { 
      if (i->second.width() > result) result = i->second.width();
    }
  return result;
}


// liste des corners en base 2
domain domain::corner(int j)
{
  domain result;
  for (map<string,interval>::const_iterator i = (*this).begin(); i!= (*this).end(); ++i)
    { 
      bool test = (j % 2);
      if (test == false) 
	result[i->first] = (*this)[i->first].inf();
      else 
	result[i->first] = (*this)[i->first].sup();
      j = (j - (j%2))/2;
    }
  return result;
}


domain domain::corner(map <string, bool> boolean_vector )
{
  domain result;
  for (map<string,bool>::const_iterator i = boolean_vector.begin(); i!= boolean_vector.end(); ++i)
    { 
      if (i->second == false) 
	  result[i->first] = (*this)[i->first].inf();
      else 
	result[i->first] = (*this)[i->first].sup();
    }
  return result;
}

void domain::out()
{
  for (map<string,interval>::const_iterator i = (*this).begin(); i!=(*this).end(); ++i)
    { 
      cout << i->first << i->second  << endl;
    }
}



domain domain::inflate(double eps)
{
  domain result;
  result = (*this);
  interval EPS(-eps,eps);
  for (map<string,interval>::const_iterator i = begin(); i!=end();++i)
      result[i->first] = i->second + EPS;
  return result;
}


int domain::dimension()
{
  int j = 0;
  for (map<string,interval>::const_iterator i = begin(); i!=end();++i)
      j++;
  return j;
}


int domain::topological_dimension()
{
  int j = 0;
  for (map<string,interval>::const_iterator i = begin(); i!=end();++i)
    if (!i->second.isPoint()) { j++; };
  return j;
}


domain domain::operator+(domain v)
{	
  domain result;  
  for (map<string,interval>::const_iterator i = begin(); i!=end();++i)
      result[i->first] = i->second + v[i->first];
  return result;
}


domain domain::intersect(domain v)
{	
  domain result;  
  for (map<string,interval>::const_iterator i = begin(); i!=end();++i)
    result[i->first] = i->second.intersect(v[i->first]);
  return result;
}


domain domain::operator-(domain v)
{	
  domain result;  
  for (map<string,interval>::const_iterator i = begin(); i!=end();++i)
      result[i->first] = i->second - v[i->first];
  return result;
}


domain domain::center()
{	
  domain result;  
  for (map<string,interval>::const_iterator i = begin(); i!=end();++i)
    result[i->first] = interval(i->second.mid());
    return result;
}




domain domain::hull(domain v)
{	
  domain result;  
  for (map<string,interval>::const_iterator i = begin(); i!=end();++i)
    result[i->first] = i->second.hull(v[i->first]);
  return result;
}


domain domain::direct_sum(domain v)
{	
  domain result = (*this);
  for (map<string,interval>::const_iterator i = v.begin(); i!=v.end();++i)
    result[i->first] = v[i->first];
  return result;
}


bool domain::disjoint(domain v)
{	
  for (map<string,interval>::const_iterator i = v.begin(); i!=v.end();++i)
    {
      if (v[i->first].disjoint((*this)[i->first])) return true;
    }
  return false;
}


domain domain::operator*(interval a)
{	
  domain result;
  result = (*this);
  for (map<string,interval>::const_iterator i = begin(); i!=end();++i)
      result[i->first] = a*i->second;
  return result;
}


domain domain::operator*(double a)
{	
  domain result;
  interval A = interval(a,a);
  result = (*this);
  for (map<string,interval>::const_iterator i = begin(); i!=end();++i)
      result[i->first] = A*i->second;
  return result;
}


bool domain::subset(domain v)
{	
  bool result = true;
  for (map<string,interval>::const_iterator i = begin(); i!=end();++i)
      result = result && i->second.subset(v[i->first]);
  return result;
}

void domain::latex(out_latex latex, string color)
{
    ofstream dessin;
    const char* p = latex.fichier.c_str();

    dessin.open(p,ios::app);

    dessin   << "\\psline[linewidth=0.005,linecolor=" << color  << "]{-}(" ;
    dessin   << (*this)["x"].inf() << "," << (*this)["y"].inf()  << ")(";
    dessin   << (*this)["x"].sup() << "," << (*this)["y"].inf()  << ")(";
    dessin   << (*this)["x"].sup() << "," << (*this)["y"].sup()  << ")(";
    dessin   << (*this)["x"].inf() << "," << (*this)["y"].sup()  << ")(";
    dessin   << (*this)["x"].inf() << "," << (*this)["y"].inf()  << ")" << endl;
    dessin.close();
}


vector <string> domain::variables()
{
  vector <string> result;
  for (map<string,interval>::const_iterator i = (*this).begin(); i!=(*this).end(); ++i)
    { 
      result.push_back(i->first);
    }  
}


