#include "grille.h"
#include <math.h>

grille::grille(double x1, double y1, double x2 , double y2  , int t)
{
  vector_nD < double > X;
  double largeur_x = x2-x1;
  double largeur_y = y2-y1;
  double delta_x = largeur_x/ (double)t;
  double delta_y = largeur_y/ (double)t;

  vector_nD < double > X_couleur;
  X_couleur.push_back(largeur_x);
  X_couleur.push_back(largeur_y);
  double norme_max = X_couleur.norme();


  (*this).taille = t;
  for (int i = 0; i < taille+1; i++ )  
  for (int j = 0; j < taille+1; j++)
    {
      X.clear();
      X.push_back( x1 + delta_x*i  );
      X.push_back( y1 + delta_y*j  );
      (*this).push_back(X);
      
      X_couleur.clear();
      X_couleur.push_back(X[0]-x1);
      X_couleur.push_back(X[1]-y1);
      double norme = X_couleur.norme();
      (*this).color.push_back(norme/norme_max);
    };
}


grille::grille()
{

};


grille::grille(expression f1, expression f2, double x1, double y1, double x2 , double y2  , int t )
{

  vector_nD < double > Y;
  double largeur_x = x2-x1;
  double largeur_y = y2-y1;
  double delta_x = largeur_x/(double)t;
  double delta_y = largeur_y/(double)t;

  double x_1;
  double x_2;
  double y_1;
  double y_2;

  vector_nD < double > X_couleur;
  X_couleur.push_back(largeur_x);
  X_couleur.push_back(largeur_y);
  double norme_max = X_couleur.norme();
  
  (*this).taille = t;
  for (int i = 0; i < taille+1; i++)  
  for (int j = 0; j < taille+1; j++)
    {
      Y.clear();
      x_1 = x1 + delta_x*(double)i;
      x_2 = y1 + delta_y*(double)j;

      domain d;
      d["x"] = interval(x_1, x_1);
      d["y"] = interval(x_2, x_2);
      

      y_1 = f1.Evalue(d).inf();
      y_2 = f2.Evalue(d).inf();
      
      Y.push_back(y_1);
      Y.push_back(y_2);
      (*this).push_back(Y);

      X_couleur.clear();
      X_couleur.push_back(x_1-x1);
      X_couleur.push_back(x_2-y1);
      double norme = X_couleur.norme();
      (*this).color.push_back(norme/norme_max);
      
      
    };

}

grille::grille( domain P , double size)
{
  double x1 = P["x"].inf();
  double x2 = P["x"].sup();
  double y1 = P["y"].inf();
  double y2 = P["y"].sup();
  
  vector_nD < double > X;
  double largeur_x = x2-x1;
  double largeur_y = y2-y1;
  double delta_x = largeur_x/size;
  double delta_y = largeur_y/size;

  for (int i = 0; i < size; i++ )  
  for (int j = 0; j < size; j++)
    {
      X.clear();
      X.push_back( x1 + delta_x*(double)i  );
      X.push_back( y1 + delta_y*(double)j  );
      (*this).push_back(X);
    };

}


// en fonction d'un nombre entre 0 et 1 cette focntion renvoie la kieme composante une couleur
double couleur(int k, double t)
{
double result;
if ((t<0.5) && (k==1)) return 1;
if ((t<0.5) && (k==2)) return 2*t;
if ((t<0.5) && (k==3)) return 0;
if ((t>=0.5) && (k==1)) return -2*t+2;
if ((t>=0.5) && (k==2)) return -2*t+2;
if ((t>=0.5) && (k==3)) return 2*t-1;
return result;
}


void grille::dessine()
{
  double a = 0;

for (int k = 0; k <  (*this).taille+1 ; k++ )  
 for (int j = 0; j <  (*this).taille ; j++ )  
      { 
	double r = 0.1;
	glBegin(GL_LINE_STRIP);
	double t = 1-(*this).color[k*(taille+1)+j];
	glColor3f(couleur(1,t), couleur(2,t),  couleur(3,t));
	double x = ((*this)[k*(taille+1)+j])[0];
	double y = ((*this)[k*(taille+1)+j])[1];
	glVertex3d(x,y,a);
	double xb = ((*this)[k*(taille+1)+j+1])[0];
	double yb = ((*this)[k*(taille+1)+j+1])[1];
	glVertex3d(xb,yb,a);
	glEnd() ;
      }


for (int k = 0; k <  (*this).taille ; k++ )  
 for (int j = 0; j <  (*this).taille+1 ; j++ )  
      { 
	double r = 0.1;
	glBegin(GL_LINE_STRIP);
	double t = 1-(*this).color[k*(taille+1)+j];
	glColor3f(couleur(1,t), couleur(2,t),  couleur(3,t));
	double x = ((*this)[k*(taille+1)+j])[0];
	double y = ((*this)[k*(taille+1)+j])[1];
	glVertex3d(x,y,a);
	double xb = ((*this)[k*(taille+1)+j+taille+1])[0];
	double yb = ((*this)[k*(taille+1)+j+taille+1])[1];
	glVertex3d(xb,yb,a);
	glEnd() ;
      }
}



void grille::dessine_latex(string lala)
{
   out_latex dessin3;
   std::ostringstream oss;
   dessin3.fichier = lala;
   dessin3.initialise();

   ofstream dessin;
   const char* p = dessin3.fichier.c_str();
   
   dessin.open(p,ios::app);
   dessin.precision(8);
      
   for (int k = 0; k <  (*this).taille+1 ; k++ )  
     for (int j = 0; j <  (*this).taille ; j++ )  
       { 
	 double r = 0.1;
	 double t = 1-(*this).color[k*(taille+1)+j];
	 double t1, t2, t3;
	 t1 = couleur(1,t);
	 t2 = couleur(2,t);
	 t3 = couleur(3,t);
	 dessin << fixed << "\\newrgbcolor{la}{"<< t1 << " " << t2 << " " << t3 << "}" << endl  ; 
	 double x = ((*this)[k*(taille+1)+j])[0];
	 double y = ((*this)[k*(taille+1)+j])[1];
	 double xb = ((*this)[k*(taille+1)+j+1])[0];
	 double yb = ((*this)[k*(taille+1)+j+1])[1];
	 dessin << fixed << "\\psline[linewidth=0.01, linecolor=la]{-}(" << x  << "," <<  y-1 << ")" ;
	 dessin << fixed << "(" << xb  << "," <<  yb -1<< ")" << endl << endl ; 
	}

   for (int k = 0; k <  (*this).taille ; k++ )  
     for (int j = 0; j <  (*this).taille+1 ; j++ )  
       { 
	 double r = 0.1;
	 double t = 1-(*this).color[k*(taille+1)+j];
	 double t1, t2, t3;
	 t1 = couleur(1,t);
	 t2 = couleur(2,t);
	 t3 = couleur(3,t);
	 dessin << fixed << "\\newrgbcolor{la}{"<< t1 << " " << t2 << " " << t3 << "}" << endl  ; 
	 double x = ((*this)[k*(taille+1)+j])[0];
	 double y = ((*this)[k*(taille+1)+j])[1];
	 double xb = ((*this)[k*(taille+1)+j+taille+1])[0];
	 double yb = ((*this)[k*(taille+1)+j+taille+1])[1];
	 dessin << fixed << "\\psline[linewidth=0.01, linecolor=la]{-}(" << x  << "," <<  y-1 << ")" ;
	 dessin << fixed << "(" << xb  << "," <<  yb -1 << ")" << endl << endl ; 
       }
       
    dessin.close();

   dessin3.fin();
}



void grille::cl()
{
  int t = (*this).size();
    for (int i = 0; i < t; i++ )  
      (*this)[i].clear();
    (*this).clear();
}

