#include <stdlib.h>
#include <SDL.h>
#include <SDL_opengl.h>

#include "expression.h"
#include "grille.h"
#include "thom.h"
#include "mesh.h"
#include "map.h"
#include "newton.h"
#include "connexe.h"
#include "vector_field.h"

#include "sivia.h"


// Entry point
int main(int argc, char *argv[])
{

  //graphics initialisation
  int j0 = 0;
  double zoom = 0.2;
  double co_zoom = 0.2;

  float x_position, y_position ;

  double Inflate = 0.001;

  float co_x_position, co_y_position, co_xVel, co_yVel;

  x_position=0; y_position=0; 
  co_x_position=0; co_y_position=0; co_xVel=0; co_yVel=0;

  bool source = true;
  bool filtre = false;
  bool contre = false;
  bool inflate = false;

  int bord_i=0;

  SDL_Init(SDL_INIT_VIDEO);
  SDL_WM_SetCaption("Thom - Version 3", NULL);
  
  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
  SDL_SetVideoMode(1000, 1000, 32, SDL_OPENGL);
  //graphics initialisation end


  //point fronce avec un bord générique
  expression f1("x+0.2*y");
  expression f2("y^3-x*y");
  double x_min = -0.2;
  double x_max = 0.33;
  double y_min = -0.7;
  double y_max = 0.61;


  /*exemple du robot
  expression f1("2*cos(x)+cos(x+y)");
  expression f2("2*sin(x)+sin(x+y)");
  double x_min = -2;
  double x_max = 2;
  double y_min = -2;
  double y_max = 3;
  */

 /*exemple sympa avec une boucle et des tours*/

  /*   expression f1("0.5*x + (1-y^2)*cos(x + 0.2*y) ");
   expression f2("(1-y^2)*sin(x)");
   double x_min = -1;
   double x_max = 4;
   double y_min = -0.4;
   double y_max = 0.6;
  */

  //exemple sympa avec une boucle sans tours

  /* expression f1("0.5*x + (1-y)*cos(x + 0.2*y) ");
   expression f2("(1-y)*sin(x)");
   double x_min = -1;
   double x_max = 4;
   double y_min = -0.2;
   double y_max = 0.3;
  */

  // declaration grille affiché
  int nb = 25;
  


  grille gx(x_min, y_min, x_max, y_max,nb);
  grille gy(f1,f2, x_min, y_min, x_max, y_max,nb);
  
  gx.dessine_latex("output/source.tex");
  gy.dessine_latex("output/target.tex");


  domain dom;
  dom["x"] = interval(x_min,x_max);
  dom["y"] = interval(y_min,y_max);
  
  vector <expression> l2;
  l2.push_back(f1);
  l2.push_back(f2);
  
  Map nico2(l2);
   
  mesh m(dom,nb) ;
  
  thom essai(f1,f2, m);

  
  domain zero;
  zero["y0"] = interval(0);
  
  mesh boundary = m.boundary();
  
  // applique la fonction one to one a chaque pavé de la grille
  essai.one_to_one();
  
  // calcul l'image de chaque élément de la grille 
  essai.image();  
  
  essai.critical_points(dom);  
  
  essai.relation();
  essai.relation2();
  
  //  essai.remove_critical_points();
  
  // essai.courbe_singuliere(0.001);
  
  /*  GLdouble ray_x, ray_y, ray_z;
      GLint viewport[4];
      GLdouble proj[16];
      GLdouble modelview[16];
      
      // we query OpenGL for the necessary matrices etc.
      glGetIntegerv(GL_VIEWPORT, viewport);
      glGetDoublev(GL_PROJECTION_MATRIX, proj);
      glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
  */
  double x =0;
  double y =0;
  
  while (1)
    {
      co_x_position = co_x_position + co_xVel ;
      co_y_position = co_y_position + co_yVel ;

      glClearColor(0,0,0,0);
      glClear(GL_COLOR_BUFFER_BIT);
      
      glPushMatrix();

      glPushMatrix();

      glScalef(zoom,zoom,zoom);
      glTranslatef(x_position,y_position,0);
      
     essai.res.dessine();  

     //  essai.res.dessine_bord();            
     //  essai.res.dessine(j0); 

     if (contre) { essai.res.dessine_contre_image_d_image(j0); }
     if (filtre) { essai.res.dessine_contre_image_de_singularite(); }

      
//      essai.res.dessine_contre_image_du_bord(bord_i);
     //if (filtre) { essai.res.dessine_contre_image_du_bord(); essai.res.dessine_contre_image_de_singularite();}
     //essai.res.dessine_fr_cc_contre_image_du_bord(bord_i);
      
     gx.dessine();
     essai.res.dessine_L_cur();  
     essai.res.dessine_courbe_singuliere();  
     
     
     // essai.res.dessine_boundary_boundary_v( m  );
     /*essai.res.dessine_boundary_boundary();
     essai.res.dessine_singular_singular_v();
     */
     essai.res.dessine_singular_boundary_v( Inflate  );
     essai.res.dessine_cusp_v( Inflate );
     
     
      GLdouble model_view[16];
      glGetDoublev(GL_MODELVIEW_MATRIX, model_view);
      GLdouble projection[16];
      glGetDoublev(GL_PROJECTION_MATRIX, projection);
      GLint viewport[4]; 
      glGetIntegerv(GL_VIEWPORT, viewport);
      double dx; double dy; double dz;
      GLfloat depth[2];	 
      glReadPixels (x, y, 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, depth); 
      gluUnProject(x, viewport[3] - y, depth[0], model_view, projection, viewport, &dx, &dy, &dz);
      float x3d = float(dx);
      float y3d = float(dy);
      float z3d = float(dz);

      float depart_x;
      float depart_y;
     
      glPopMatrix();
      
      glPushMatrix();
      glScalef(co_zoom,co_zoom,co_zoom);
      glTranslatef(co_x_position,co_y_position,0);
      
      essai.res.dessine_image(j0);      
      gy.dessine();

      GLdouble co_model_view[16];
      glGetDoublev(GL_MODELVIEW_MATRIX, co_model_view);
      GLdouble co_projection[16];
      glGetDoublev(GL_PROJECTION_MATRIX, co_projection);
      GLint co_viewport[4]; 
      glGetIntegerv(GL_VIEWPORT, co_viewport);
      double co_dx; double co_dy; double co_dz;
      GLfloat co_depth[2];	 
      glReadPixels (x, y, 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, depth); 
      gluUnProject(x, co_viewport[3] - y, co_depth[0], co_model_view, co_projection, co_viewport, &co_dx, &co_dy, &co_dz);
      float co_x3d = float(co_dx);
      float co_y3d = float(co_dy);
      float co_z3d = float(co_dz);

      float co_depart_x;
      float co_depart_y;


      glPopMatrix();      
      glPopMatrix();
      
      SDL_GL_SwapBuffers();
      
      domain X;
      
      SDL_Event event;
      while (SDL_PollEvent(&event))
	{
	  switch (event.type)
	    {
	    case SDL_MOUSEBUTTONDOWN:
	      if (event.button.button == SDL_BUTTON_WHEELDOWN)
		{
		if (inflate) (Inflate = Inflate/1.1);
		if (source && !inflate) zoom=zoom/1.1;
		if (!source && !inflate) co_zoom=co_zoom/1.1 ; 	        
	        }
	      if (event.button.button == SDL_BUTTON_WHEELUP)
		{
		  if (inflate) (Inflate = Inflate*1.1);		  
		  if (source && !inflate) zoom=zoom*1.1;
		  if (!source && !inflate) co_zoom=co_zoom*1.1 ; 	        
		}
	      if (event.button.button ==  SDL_BUTTON_LEFT)
		{
		  if (source)
		    {
		      depart_x = x3d;
		      depart_y = y3d;		 
		    }
		  else
		    {
		      co_depart_x = co_x3d;
		      co_depart_y = co_y3d;		 
		    }
		    
		}
	      break;
	      
	     case SDL_MOUSEBUTTONUP:
		{	      
		  if (event.button.button ==  SDL_BUTTON_LEFT)
		    {
		      if (source)
			{
			x_position = x_position + x3d - depart_x ;
			y_position = y_position + y3d - depart_y ;		     
			}
		      else
			{
			co_x_position = co_x_position + co_x3d - co_depart_x ;
			co_y_position = co_y_position + co_y3d - co_depart_y ;		     
			}
			
     		    }
		}
		break;
	      
	    case SDL_MOUSEMOTION:
	      x = event.motion.x;
	      y = event.motion.y;
	      X["x"] = interval(x3d,x3d);
	      X["y"] = interval(y3d,y3d);
      	      j0 = essai.res.mesH.which_one(X);
	      break;
	      
	    case SDL_KEYUP:
	      switch( event.key.keysym.sym )
		{
		case SDLK_ESCAPE: return(0); break;
		default: break;
		}
	      break;

	    case SDL_KEYDOWN:
	      switch( event.key.keysym.sym )
		{
		case SDLK_s : source = true; break;
		case SDLK_t : source = false; break;
		case SDLK_f : filtre = !filtre; break;
		case SDLK_c : contre = !contre; break;
		case SDLK_i : inflate = !inflate; break;
		  /*		case SDLK_i : bord_i = bord_i + 1 ;  cout << bord_i << endl; break;
				case SDLK_o : bord_i = bord_i - 1 ; break;*/
		default: break;
		}
	      break;

	    case SDL_QUIT:  return(0);
	    default: break;
	    }
	}
      SDL_Delay(20);
      }
  return 0;
}



