#include <list>
#include <stack>

#include "thom.h"


// modifie la fonction f en enlevant les variables qui sont des constantes
int equation (Map & f, domain & d)
{
  vector < string > to_be_erased;
  for (domain::const_iterator i = d.begin(); i!=d.end();++i)
    {
      interval t = i->second;
      if (t.isPoint()) 
	{  
	  std::set<string>::const_iterator dep;
	  for(  dep = f.dependent_variable.begin() ; dep!= f.dependent_variable.end() ;++dep) 
	    {
	      expression lm =  f.get_component(*dep).substitute( t.inf() , i->first );
	      f.set_component(*dep,lm);
	    }
	  to_be_erased.push_back(i->first);	      
	}
    }
  
  for (int j = 0; j < to_be_erased.size(); j++)
    {
      d.erase(to_be_erased[j]);
    }

}



// Constructeur
thom::thom(expression function1, expression function2, mesh m)
{
  
  res.mesH = m;
  res.boundary = res.mesH.boundary();

  initial = m.initial;

  vector < expression > F;
  F.push_back(function1);
  F.push_back(function2);
 
  f = Map(F) ;
  df = f.differentielle();

  //  df.out();

  // crée une expression qui code la différentielle de f
  (*this).determina = df.determinant();
  F.clear();
  F.push_back(determina);
  Singular_curve = Map(F);

  // Singular_curve.out();
  gradient = Singular_curve.differentielle();


  // création la map xi qui code le champ de vecteur contenant 
  vector <expression> tr2;
  tr2.push_back(expression("-1")*gradient.coefficient["y0"]["y"]);
  tr2.push_back(gradient.coefficient["y0"]["x"]);

  xi = Map(tr2);
  xi.independent_variable.insert("x");
  xi.independent_variable.insert("y");

  // xi.out();


  // création la map qui permet de tester l'intersection de courbe singulière dans l'espace d'arrivée
  vector <expression> tr;
  tr.push_back(function1.substitute("x1","x").substitute("x2","y") - function1.substitute("x3","x").substitute("x4","y"));
  tr.push_back(function2.substitute("x1","x").substitute("x2","y") - function2.substitute("x3","x").substitute("x4","y"));
  tr.push_back(determina.substitute("x1","x").substitute("x2","y"));
  tr.push_back(determina.substitute("x3","x").substitute("x4","y"));

  Map f3(tr);
  newton_singular_singular = newton(f3);
  //f3.out();

  vector < expression > F_cusp;

  expression fisrt_component = df.coefficient["y0"]["x"]*gradient.coefficient["y0"]["y"] - df.coefficient["y0"]["y"]*gradient.coefficient["y0"]["x"];
  F_cusp.push_back(fisrt_component);
  expression second_component = df.coefficient["y1"]["x"]*gradient.coefficient["y0"]["y"] - df.coefficient["y1"]["y"]*gradient.coefficient["y0"]["x"];
  F_cusp.push_back(second_component);
  cusp = Map(F_cusp);
  newton_cusp = newton(cusp);

  cusp.out();
 
  Map vector_field1 ; 
  vector_field1.set_component("x",xi.get_component("y0"));
  vector_field1.set_component("y",xi.get_component("y1"));
  
  /*  third_derivative = f.derive(vector_field1,0);
  third_derivative.out();

  expression one_cusp_ex  = expression("a") * third_derivative.get_component("y0") + expression("b") * third_derivative.get_component("y1") ;
  one_cusp = Map(one_cusp_ex);
  one_cusp.independent_variable.insert("a");
  one_cusp.independent_variable.insert("b");
  one_cusp.independent_variable.insert("x");
  one_cusp.independent_variable.insert("y");
  */

  // one_cusp.out();  

  vector_field xi2; 
  xi2.expr2["x"] = xi.get_component("y0");
  xi2.expr2["y"] = xi.get_component("y1");

  //  xi2.out();

  /*  domain zero;
  zero["x"] = interval(0);
  zero["y"] = interval(0);
  */

  /*  xi2.Taylor(zero,100).out();
  f.derive(xi2,100,zero).out();
  */

};




// Sorte de méthode principale qui cherche les points critiques :
//  
bool thom::critical_points(domain X)
{


  cout << "Beginning of the procedure which find the critical points" << endl;

  // Méthode qui cherche les points cusp 
  // Cas numéro 1 de l'article

   int cusp_unknown = Cusp(X);
  //  int cusp_unknown = 0;

  improve_cusp();
  if (cusp_unknown==0) { cout << "     The number of cusp points is exactly : " <<   res.cusp_v.size()  <<  endl;   }
  else cout << "     The algorithm cusp was not able to determine how many cusp it exists" << endl;

  // Méthode qui cherche l'intersection de singularités dans l'image
  // Cas numéro 2 de l'article
  singular_singular(X, false);
  cout << "     The number of transversal intersection of fold curves in the target is exactly : " << res.singular_singular_v.size() << endl;


  // Méthode qui cherche l'intersection de bord dans l'image
  // Cas numéro 3 de l'article
  boundary_boundary(X, false);
  cout << "     The number of transversal intersection of boundary curves in the target is exactly : " << res.boundary_boundary_v.size() << endl;

  // Méthode qui cherche l'intersection de singularités dans l'image
  // Cas numéro 4 de l'article
  singular_boundary(X, false);
  cout << "     The number of intersection fold curves with boundary is exactly : " << res.singular_boundary_v.size() << endl;  
  improve_singular_boundary();

  cout << "End of the procedure which find the critical points" << endl;


  cout << "Beginning of the procedure that check the topology of the singular curve" << endl;
  if (relation4()==true) 
    { 
      cout << " Failure  "   <<  endl; 
    }
  else 
    {
      cout << " The singular curve is known. "   <<  endl; 
    }
  cout << "End the procedure that check the topology of the singular curve" << endl;
};


domain thom::evalue(domain P)
{
  domain result2;
  result2 = f.evalue(P); 
  return result2;
};


// test si la fonction est injective sur le domaine x
iboolean thom::one_to_one(domain x)
  {
    interval jac = df.evalue(x).determinant();
    double zero = 0;
    bool result = jac.contains(zero);
    if (result==false) return itrue;
    else return iperhaps;
  }


// test si la fonction restreinte au bord est injective
bool thom::one_to_one_boundary(domain boundary)
{
  bool result = true ;
  domain zero;   
  // génére le vecteur nul
  std::set<string>::const_iterator dep;
  for(  dep = f.dependent_variable.begin() ; dep!= f.dependent_variable.end() ;++dep) 
    {
      zero[*dep] = interval(0);
    }

  domain partial_derivative;
  for (domain::const_iterator i = boundary.begin(); i!=boundary.end();++i) 
    {
   interval t = i->second;
    if (! t.isPoint()) 
	{  
	  partial_derivative = df.evalue(boundary).row(i->first);
	}
    }
  result = (!zero.subset(partial_derivative));
  return result;    
}



// test si la fonction restreinte au à la singularité est injective
bool thom::one_to_one_singular(domain x)
  {
    if (one_to_one(x)==itrue) return true;
    
    bool result = true ;
    domain zero;   
    // génére le vecteur nul
    std::set<string>::const_iterator dep;
    for(  dep = f.dependent_variable.begin() ; dep!= f.dependent_variable.end() ;++dep) 
      {
	zero[*dep] = interval(0);
      }
    domain gradient_x;
    gradient_x["x"] = gradient.evalue(x).c["y0"]["y"];
    gradient_x["y"] = interval(-1)*gradient.evalue(x).c["y0"]["x"] ;

    /*    gradient_x.out();    
    df.evalue(x).image(gradient_x).out(); 
    */
    result = (!zero.subset(df.evalue(x).image(gradient_x)));
    return result || one_to_one_cusp(x,false);
  }


bool thom::one_to_one_boundary(domain boundary1, domain boundary2 )
  {
    bool result;
    domain zero;   
    domain intersection = boundary1.intersect(boundary2);
    domain partial_derivative1, partial_derivative2;

    std::set<string>::const_iterator dep;
    for(  dep = f.dependent_variable.begin() ; dep!= f.dependent_variable.end() ;++dep) 
      {
	zero[*dep] = interval(0);
      }

    for (domain::const_iterator i = boundary1.begin(); i!=boundary1.end();++i)
      {
	interval t = i->second;
	if (! t.isPoint()) 
	  {  
	    if ((intersection[i->first] - boundary1.center()[i->first]).inf() > 0) 
	      partial_derivative1 = df.evalue(boundary1).row(i->first);
	    else 
	      partial_derivative1 = df.evalue(boundary1).row(i->first) * (-1);
	  }
      }

    for (domain::const_iterator i = boundary2.begin(); i!=boundary2.end();++i)
      {
	interval t = i->second;
	if (! t.isPoint()) 
	  {  
	    if ((boundary2.center()[i->first] - intersection[i->first]).inf() > 0)
	      partial_derivative2 = df.evalue(boundary2).row(i->first);
	    else 
	      partial_derivative2 = df.evalue(boundary2).row(i->first) * (-1);
	  }
      }

    domain partial_derivative = partial_derivative1.hull(partial_derivative2) ;
    result = !zero.subset(partial_derivative);
    result = result || (one_to_one(boundary1.hull(boundary2)) == itrue) ;
    
    return result;    
  }





// calcule une sur évaluation de l'image de chaque élément et stocke
// le résultat dans les vectors image et image_boundary
void thom::image()
{  

  mesh::iterator it;
  for (it = res.mesH.begin(); it!=res.mesH.end();++it)
    {
      domain dom =  *it;
      domain codom ;
      codom = (*this).evalue(dom);
      res.image.push_back(codom);
    }
  
  for (it = res.boundary.begin(); it!=res.boundary.end();++it)
    {
      domain dom =  *it;
      domain codom ;
      codom = (*this).evalue(dom);
      res.image_boundary.push_back(codom);
    }  
}


// remplit un vecteur de iboolean qui sait si f est injective
void thom::one_to_one()
{

  mesh::iterator it;
  int k=0;
  for (it = res.mesH.begin(); it!=res.mesH.end();++it)
    {      
      domain dom =  *it;
      domain zero;
      zero["y0"] = interval(0);
      if (zero.subset(Singular_curve.evalue(dom))) 
	{
	  res.singular_curve.push_back(k);
	}
      k++;
    }


}


void thom::courbe_singuliere(double epsilon)
{

  domain zero2;
  zero2["y0"] = interval(0);
  sivia courbe = sivia(Singular_curve,zero2);
  courbe.SIVIAfile(initial,epsilon);
  res.courbe_singuliere = courbe.L_sivia_MB;

}

/*void thom::singular_curve(mesh m)
  {
    
    list < domain > singular_curve ; 
    mesh::iterator it;
    for (it = m.begin(); it!=m.end();++it)
      {
	singular_curve.push_back(*it);
      }
    int n = 0;
    while( n < 1000)
      {
	
      }
  }
*/


// relation, deux pavés i et j sont en relation si leurs images (via
// le calcul par intervalle) sont d'intersection non vide.
// vérifie aussi si un domain a son image qui intersecte l'image d'un bord
void thom::relation()
{  
  int taille = res.mesH.size();
  int i;
  for (i = 0; i < taille; i++)
    {
      vector < int > liste; 
      int j = 0;
      for (j = 0; j < taille; j++)
	{
	  if (res.image[i].disjoint(res.image[j]) == false ) 
	    {
	      liste.push_back(j);
	    }
	}
      res.contre_image_d_image.push_back(liste);
      /*      res.composante_connexe_contre_image_d_image.push_back(composantes_connexes(liste));
      res.fr_composante_connexe_contre_image_d_image.push_back(fr_composantes_connexes(i));
      */

      bool _contre_image_de_bord = false;
      int taille2 = res.image_boundary.size();
      for (j = 0; j < taille2; j++)
	{
	  if (res.image[i].disjoint(res.image_boundary[j]) == false ) 
	    { 
	      _contre_image_de_bord = _contre_image_de_bord || true;
	    }
	}
      res.contre_image_de_bord.push_back(_contre_image_de_bord);
      //      cout << i << endl;
    }
}


// Relation binaire de (boundary \times mesH); ie un graphe biparti
// codé dans un vector qui renvoie la liste des éléments de mesH en
// relation avec un élément de boundary.
void thom::relation2()
{  
      int taille2 = res.boundary.size();
      int taille = res.mesH.size();
      for (int j = 0; j < taille2; j++)
	{
	  vector < int > liste;
	  for (int i = 0; i < taille; i++)
	    {
	      if (res.image[i].disjoint(res.image_boundary[j]) == false ) 
		{ 
		  liste.push_back(i);
		}
	    }
	  res.contre_image_du_bord.push_back(liste);
	  /*	  res.composante_connexe_contre_image_du_bord.push_back(composantes_connexes(liste));
	  res.fr_composante_connexe_contre_image_du_bord.push_back(fr_composantes_connexes(j));
	  res.newton.push_back(newton_index(j));*/
	}

      taille2 = res.singular_curve.size();
      for (int j = 0; j < taille2; j++)
	{
	  vector < int > liste;
	  for (int i = 0; i < taille; i++)
	    {
	      if (res.image[i].disjoint(res.image[res.singular_curve[j]]) == false ) 
		{ 
		  liste.push_back(i);
		}
	    }
	  res.contre_image_de_singular.push_back(liste);
	  /*	  res.composante_connexe_contre_image_du_bord.push_back(composantes_connexes(liste));
	  res.fr_composante_connexe_contre_image_du_bord.push_back(fr_composantes_connexes(j));
	  res.newton.push_back(newton_index(j));*/
	}
}


// relation qui permet d'avoir une parfaite connaissance de la topologie de la courbe singulière.
bool thom::relation4()
{
  bool failure; // This is the returned value
  vector < vector < int > > voisin; 
  int taille2 = res.singular_curve.size();
  for (int j = 0; j < taille2; j++)
    {
      domain A = res.mesH[res.singular_curve[j]];
      /*      cout << "traitement du pavé : " << endl;
	      A.out();*/
      int card_voisin = 0;
      for (int i = 0; i < taille2; i++)
	{
	  if (i!=j)
	    {
	      domain B = res.mesH[res.singular_curve[i]];
	      if (!A.disjoint(B))
		{
		  domain C = A.intersect(B);
		  //cout << C.topological_dimension() << endl;
		  if (C.topological_dimension() == 0  ) 
		    {
		      domain zero;
		      zero["y0"] = interval(0);
		      if (zero.subset(Singular_curve.evalue(C))) 
			{
			  failure = true;
		          cout << "mince" << endl;
			}
		    }
		  if (C.topological_dimension() == 1  ) 
		    {
		      // C.out();
		      Map Singular_curve_temp = Singular_curve;
		      domain C_temp = C;
		      // Singular_curve_temp.out();
		      equation(Singular_curve_temp,C_temp);
		      // Singular_curve_temp.out();
		      //		      C.out();
		      //		      C_temp.out();
		      //		      cout << "------" << endl; 
		      if ( (Singular_curve_temp.evalue(C_temp.corner(0))["y0"]*Singular_curve_temp.evalue(C_temp.corner(1))["y0"]).sup() < 0 ) 
			{
			  /*
			  Singular_curve_temp
			  domain gradient_p0;
			  gradient_p0["x"] = gradient.coefficient["y0"]["x"].Evalue(C);
			  gradient_p0["y"] = gradient.coefficient["y0"]["y"].Evalue(C);
			  cout << "gradient " << gradient_p0[C_temp.begin()->first] << endl;
			  if (! gradient_p0[C_temp.begin()->first].contains(0)) { card_voisin++; cout << "++ " << endl; } 
			  else { failure = true; cout << "mince"; }*/
			  connexe con(Singular_curve_temp.component["y0"]);
			  if (con.star_shaped(C_temp)) {card_voisin++;} else { failure = true; }
  			}   
		      else 
			{
			  domain zero2;
			  zero2["y0"] = interval(0);
			  sivia courbe = sivia(Singular_curve_temp,zero2);
			  courbe.SIVIAfile(C_temp, 0.01*C_temp.width());
			  //cout << "taille des inconnues : " << courbe.L_sivia_MB.size() << endl;
			  if (courbe.L_sivia_MB.size() !=0) { cout << "mince" << endl; }
			}
		    }
		}
	    }
	}
      if (card_voisin > 0) 
	{res.pure_singular_curve.push_back(res.singular_curve[j]);}
      //      cout << "------" << card_voisin << endl;
	}
  return failure;
}


bool thom::remove_critical_points()
{

  vector <int> pure_singular_curve2;
  int n = res.pure_singular_curve.size();
  for (int i = 0; i<n;i++)
    {
      domain temp = res.mesH[res.pure_singular_curve[i]];
      bool a_enlever = false;    
      for (int j = 0 ; j < res.cusp_v.size() ; j++)
	{ 
	  a_enlever = a_enlever || res.cusp_v[j].subset(temp) ;
	}
      if (!a_enlever) pure_singular_curve2.push_back(res.pure_singular_curve[i]);
    }
  res.pure_singular_curve = pure_singular_curve2;
};








/*
bool thom::relation3()
{
  int taille2 = res.boundary.size();
  for (int i = 0; i < taille2; i++)
    {
      for (int j = 0; j < taille2; j++)
	{
	  //	  cout << i << " | "  << j << " resultat :"  << relation3(i,j) << endl; 
	  if (i <= j)
	    {
	      int etat = boundary_boundary(res.boundary[i],res.boundary[j], false);
	      //      	      int etat = 0;
	      if (i==j){ 	      cout << i << " | "  << j << " resultat :"  << etat << endl; }
	      pair < domain, domain> Py = make_pair(res.boundary[i],res.boundary[j]);
	      pair < pair <domain, domain>, int > add (Py,etat);
	      res.boundary_boundary_v.push_back(add);
	    }
	  // res.boundary_boundary_v.push_back(make_pair(res.boundary[i],res.boundary[j])); 
	}
    }
}
*/





pair <domain , domain > thom::improve_boundary_boundary(pair <domain , domain>  Bi)
{

  // contruction du domaine B
  domain B;
  B["x1"] = Bi.first["x"] ;
  B["x2"] = Bi.first["y"];
  B["x3"] = Bi.second["x"];
  B["x4"] = Bi.second["y"];

  domain copie_B = B;

  vector <expression> tr;
  expression f1 = f.get_component("y0");
  expression f2 = f.get_component("y1");
  tr.push_back(f1.substitute("x1","x").substitute("x2","y") - f1.substitute("x3","x").substitute("x4","y"));
  tr.push_back(f2.substitute("x1","x").substitute("x2","y") - f2.substitute("x3","x").substitute("x4","y"));
  Map f_temp(tr);
  equation(f_temp,B);
  newton n(f_temp);
  domain x = n.operator_newton(B);
  
  domain result1;
  domain result2;

  for (domain::const_iterator i = B.begin(); i!=B.end();++i)
    {
      interval t = i->second;
      if (t.isPoint()) 
	{  
	  copie_B[i->first] = copie_B[i->first] ;
	}
      else 
	{  
	  copie_B[i->first] = x[i->first] ;
	}

    }
  
  result1["x"] = copie_B["x1"];
  result1["y"] = copie_B["x2"];

  result2["x"] = copie_B["x3"];
  result2["y"] = copie_B["x4"];


  return make_pair(result1,result2);

}



// (b_1 \times b_2) si on a la preuve qu'il existe 
// un point unique a_1 \in b_1 et unique a_2 \in b_2 tels que f(a_1) = f(a_2)
// ATTENTION : s'appuie sur la fonction "equation"

// valeur retournée : 
// 1 b1 et b2 sont indentiques et la fonction est injective sur ce domaine
// 2 les deux images ne s'intersectent pas du tout
// 3 les deux pavés initiaux sont voisins et les images ne s'intersectent qu'en un point car leur enveloppe est injective (donc ne s'intersecte pas transversalement)
// 4 newton montre qu'il n'y pas d'intersection
// 5 montre qu'il existe une unique solution
// 6 et 7 on ne peut rien conclure
// 8 un des deux pavés n'est pas full rank -> reste a traiter
int thom::boundary_boundary(domain B1, domain B2 , bool verbose )
{
  if (verbose) {B1.out() ; B2.out();}

  if ((B1 == B2) && (one_to_one_boundary(B2))) return 1;
  if (!( one_to_one_boundary(B2) && one_to_one_boundary(B1) )) return 8;
  if (evalue(B1).disjoint(evalue(B2)) ) return 2;
  if (B1.disjoint(B2)==false ) 
    {
    if  (one_to_one_boundary(B1,B2))
	 return 3;
       else 
	 return 7;
    }

    // contruction du domaine B
  domain B;
  B["x1"] = B1["x"] ;
  B["x2"] = B1["y"];
  B["x3"] = B2["x"];
  B["x4"] = B2["y"];

  vector <expression> tr;
  expression f1 = f.get_component("y0");
  expression f2 = f.get_component("y1");
  tr.push_back(f1.substitute("x1","x").substitute("x2","y") - f1.substitute("x3","x").substitute("x4","y"));
  tr.push_back(f2.substitute("x1","x").substitute("x2","y") - f2.substitute("x3","x").substitute("x4","y"));
  Map f(tr);
  if (verbose) {f.out();}

  equation(f,B);
  newton n(f);

  if (verbose) 
    {
      B.out();
      f.out();
    }

  domain x = B;
 
  bool finish = false;
  int nb = 0;
  while (finish == false)
    {
      x = n.operator_newton(x);
      if (verbose) {x.out();}
      nb++;
      finish = (nb > 5) || x.subset(B) || x.disjoint(B);
    }
  
  if (x.disjoint(B)) return 4;
  if (x.subset(B)) return 5; else return 6;

}



// 1 la fonction est injective sur ce pavé
// 2 les deux pavés ont des images d'intersection vide
// 9 on n'en sait rien
int thom::singular_singular(domain B1, domain B2)
{

  if ((B1 == B2) && (one_to_one_singular(B2))) return 1;
  if (B1 != B2)
    {
      //      if (one_to_one(B1)==itrue && (one_to_one(B2) == itrue))  return 1;
      if (evalue(B1).disjoint(evalue(B2)) ) return 2;
      if (B1.disjoint(B2)==false ) 
	if  ( one_to_one_singular(B1.hull(B2)))
	  return 3;
	else 
	  return 7;
      // contruction du domaine B
      domain B;
      B["x1"] = B1["x"] ;
      B["x2"] = B1["y"];
      B["x3"] = B2["x"];
      B["x4"] = B2["y"];      

      domain x = newton_singular_singular.operator_newton(B);
      
      for (int lala=0 ;  (lala < 10) && !(x.disjoint(B))  && (!x.subset(B))  ;lala++)
	{
	  x = newton_singular_singular.operator_newton(x);
	}
      if (x.disjoint(B)) return 4;
      if (x.subset(B)) return 5; else return 6;      
    }
  else 
    return 0;
}




//-------------------------------------------------------------------------------------
// Contruction de Graphe,
// i R j <=> i et j sont voisins
// input : une liste de domaines
// output : un vector contenant les arcs (i,j) avec (i \leq j)
//-------------------------------------------------------------------------------------
vector < vector < int > > thom::graphe (vector <int > in) 
{
  vector < vector < int  > > result ;
  for (int j = 0; j < in.size(); j++)
    {
      for (int k = 0; k < j; k++)
	{
	      if (res.mesH.voisin(in[j],in[k]) && in[j]!=in[k]) 
		{
		vector < int > arc;
		arc.push_back(in[j]);
		arc.push_back(in[k]);	      
		result.push_back(arc);
		}
	}
    }
  return result;
}


//-------------------------------------------------------------------------------------
// Cette algorihtme a
// - en entrée : un vector de pavés (in)
// - en sortie : la liste des composantes connexes de in avec la relation i R j si i et j sont voisins
//               ceci s'appuie sur l'algo de complexité linéaire avec les arcs.
//               result[0] sera la liste des éléments de la premières composantes         
//-------------------------------------------------------------------------------------
vector < vector  < int > > thom::composantes_connexes (vector < int > in)
{
  //----------- initialisation des composantes connexes -----------
  vector < vector < int > > result;
  for (int j = 0; j < in.size(); j++)
    {
      vector < int > temp;
      temp.push_back(in[j]);
      result.push_back(temp);
    }

  //----------- contruction de graphe -----------
  vector < vector < int > > g;
  g = (*this).graphe(in);  
  for (int i = 0; i < g.size(); i++)
    {
      vector < int > arc = g[i];
      int a = arc[0];
      int b = arc[1];
      
      int A ; // a appartient à la Aième composante connexe
      int B ; // a appartient à la Bième composante connexe
      for (int j = 0; j < result.size(); j++)
	{
	  for (int k = 0; k < result[j].size(); k++)
	    {
	      if (result[j][k]==a) A = j;
	      if (result[j][k]==b) B = j;
	    }
	}

      if (A != B)
	{
	  vector < int > cca = result[A];
	  vector < int > ccb = result[B];
	  for (int compteur = 0; compteur < ccb.size(); compteur++ )
	    {
	      cca.push_back(ccb[compteur]);
	    }
	  vector < vector < int > > temp2;
	  for (int l = 0; l < result.size(); l++)
	    {
	      if (l==A)
		temp2.push_back(cca);
	      else 
		if (l!=B)
		  temp2.push_back(result[l]);
	    }
	  result = temp2;
	  }
    }
  return result;
};

//--------------------------------------------------------------------------
//
//   test si l'enveloppe intervalle des éléments de LA composante
//   connexe de la contre image de l'élément index est partiellement
//   injective (ce qui "prouve" que tout le monde revient à sa place seulement ...)
//
//--------------------------------------------------------------------------
vector < vector  < int > > thom::fr_composantes_connexes (int index)
{
  vector < vector < int > > cc_i;
  cc_i = res.composante_connexe_contre_image_du_bord[index];
  int nb_cc = cc_i.size();
  int A = 0;
  for (int j = 0; j < nb_cc; j++)
    {
      for (int k = 0; k < cc_i[j].size(); k++)
	{
	  if (res.boundary[index].subset(res.mesH[cc_i[j][k]])) A = j;
	}
    }
  domain enveloppe = res.mesH.wrapping(cc_i[A]);
  
  if (one_to_one(enveloppe)==itrue)
    {
      vector < int > temp;
      //temp.push_back(index);
      cc_i[A] = temp;
    }
  return cc_i;
};



//--------------------------------------------------------------------------
//
//   Méthode qui remplit parcours et découpe la somme directe du bord avec lui même
//   quotienté par la relation : (x,y) = (y,x).
//
//--------------------------------------------------------------------------
void thom::boundary_boundary(domain initial , bool verbose  )
{
  vector < domain > L_cur_b = initial.boundary();
  int k=0;
  vector < pair < domain, domain > > L_courante;

  // création de la liste initiale à partir du bord
  if (verbose) {
    cout << endl << "----------------------------------------------" << endl ;
    cout << "création de la liste initiale à partir du bord" << endl ;
    cout << "----------------------------------------------" << endl ;
  } 
  int taille78 = L_cur_b.size();    
  for (int j = 0; j < taille78; j++)
    for (int i = 0; i < taille78; i++)
      {
	    domain X1 = L_cur_b[i];
	    domain X2 = L_cur_b[j];
	    if (i <= j) 
	      {
		pair < domain , domain > P = make_pair (X1,X2);		
		L_courante.push_back(P);
		if (verbose) 
		  {
		    cout << i << ","  << j << endl;
		    X1.out();
		    X2.out();
		  }
	      }
      }

  bool not_finish = true;
  while (not_finish == true)
    {
      not_finish = false; 
      vector < int > to_be_cut;
      int taille4 = L_courante.size();  
      if (verbose) 
	{
	  cout << "-------------------------------------" << endl << "Passe n° : " << k ;
	  cout << ", taille de la liste à traiter  : "<<  taille4 << endl;
	}

      for (int j = 0; j < taille4; j++)
	{
	  domain X1 = L_courante[j].first;
	  domain X2 = L_courante[j].second;
	  int etat = boundary_boundary(X1,X2, verbose);
	  if (verbose)
	    {
	      //bool egaux = (X1==X2);
	      cout << "couple n° : " << j <<  ", état : "  << etat <<   endl;
	    }
	  
	  // si cas non favorable alors futur découpage 
	  if (etat!=5 && etat!=2 && etat!=3  && etat!=1 && etat!=4 )     
	    {
	      not_finish = true;
	      to_be_cut.push_back(j);     
	    }
	  // sinon ajout à la liste 
	  else 
	    { 
	      pair <domain, domain > Py = make_pair(X1,X2);    
	      res.boundary_boundary.push_back(make_pair(Py,etat)); 
	      if (etat==5) { res.boundary_boundary_v.push_back(Py); }
	    }
	    }

      if (verbose){ cout << "Fin de la passe " << k  << ", cardinal des couples encore à traiter : " <<  to_be_cut.size() << endl ;}


      // -----------------------------------------
      // découpage des couples qui posent problème
      // -----------------------------------------
      vector < pair < domain , domain  > > Future_L_courante ; 
      
      taille4 = to_be_cut.size();  
      for (int j = 0; j < taille4; j++)
	{ 
	  int ref_couple = to_be_cut[j] ;
	  domain I = L_courante[ref_couple].first;
	  domain J = L_courante[ref_couple].second;
	  mesh Mi(I,2);
	  mesh Mj(J,2);	      	      
	  
	  if (I!=J)
	  for (int mi = 0; mi< Mi.size();mi++)
	    {
	      for (int mj = 0; mj< Mj.size();mj++)
		{
		  pair < domain , domain > P = make_pair ( Mi[mi],Mj[mj]);
		  Future_L_courante.push_back(P);
		}
	    }

	  if (I==J)
	  for (int mi = 0; mi< Mi.size();mi++)
	    {
	      for (int mj = 0; mj< Mj.size();mj++)
		{
		      if (mi <= mj)
			{
			  pair < domain , domain > P = make_pair ( Mi[mi],Mi[mj]);
			  Future_L_courante.push_back(P);
			}
		}
	    }
	}
       k++;

       L_courante.clear();
       L_courante = Future_L_courante;
       Future_L_courante.clear();
       to_be_cut.clear();

       if ( k> 15 ) 
	 {
	   int taille5 = L_courante.size();  
	   for (int j = 0; j < taille5; j++)
	     {
	       res.boundary_boundary.push_back(make_pair(L_courante[j],8));
	     }
	   not_finish = false;
          }

    }

  if (verbose)
    {
  cout << "cardinal des solutions :" << res.boundary_boundary.size();
    }
};




bool thom::one_to_one_singular_boundary(domain singular , domain boundary  , bool verbose)
{

  // teste s'il existe s dans singular et b dans boundary tel que f(s) = f(b)
  // avec s=b

  if (verbose) {
    cout << "--------------------------------------------------" << endl;
    cout << "- one to one singular ----------------------------" << endl;
    cout << "--------------------------------------------------" << endl;
  }

  Map Singular_curve_temp = Map(determina);
  if (verbose) {Singular_curve_temp.out();}
  domain boundary_copie = boundary;
  equation(Singular_curve_temp,boundary_copie);

  if (verbose) {Singular_curve_temp.out();}
  newton n(Singular_curve_temp);
  
  domain x = n.operator_newton(boundary_copie);
  x = n.operator_newton(x);
  x = n.operator_newton(x);
  x = n.operator_newton(x);

  if (verbose)  {  x.out();  }
  bool result_1 = x.subset(boundary_copie) && x.subset(singular) ;

  if (result_1 && verbose) { cout << "trop cool 1 !" << endl;}
  /*
  interval_matrix dw;
  domain gradient_x;
  gradient_x["x"] = gradient.evalue(singular).c["y0"]["y"];
  gradient_x["y"] = interval(-1)*gradient.evalue(singular).c["y0"]["x"] ;
  
  domain df_sur_ds = df.evalue(singular).image(gradient_x);

  domain df_sur_db;
  for (domain::const_iterator i = boundary.begin(); i!=boundary.end();++i) 
    {
      interval t = i->second;
      if (! t.isPoint()) 
	{  
	  df_sur_db = df.evalue(boundary).row(i->first);
	}
    }

  df_sur_ds.out();
  df_sur_db.out();

  dw.dependent_variable.push_back("y0");
  dw.dependent_variable.push_back("y1");
  dw.independent_variable.push_back("b");
  dw.independent_variable.push_back("s");

  dw.c["y0"]["b"] = df_sur_db["y0"]; 
  dw.c["y1"]["b"] = df_sur_db["y1"]; 

  dw.c["y0"]["s"] = df_sur_ds["y0"]; 
  dw.c["y1"]["s"] = df_sur_ds["y1"]; 

  dw.out();

  interval zero = interval(0);
  cout << "déterminant " << endl;
  cout << dw.determinant() << endl;
  bool result_2 = !(zero.subset(dw.determinant()));
  if (result_2) { cout << "trop cool 2 !" << endl;}*/
  return result_1 ;
}



int thom::singular_boundary(domain singular , domain boundary  , bool verbose)
{

  if (verbose) { cout << "Singular :" << endl ; singular.out() ; cout << "Boundary :" << endl ; boundary.out();}

  // test si 0 apprtient a det f (singular)
  domain zero;
  zero["y0"] = interval(0);
  if (!zero.subset(Singular_curve.evalue(singular))) return 2; // 0 not in det f (singular)
  if (evalue(singular).disjoint(evalue(boundary)) ) return 2; // f(singular) \cap f(boundary)  = \emptyset
  if (!( one_to_one_boundary(boundary) && one_to_one_singular(singular) )) return 8; // non embedding


  // if boxes intersect in source, check whether or not boundary curve intersects singular curve
  if (! singular.disjoint(boundary))
    {
      if (one_to_one_singular_boundary( singular ,  boundary  ,  verbose))
	return 7;
      else return 8;
    }


  // suppose boxes do not intersect in source, check if there image intersect in target with interval newon method
  domain B;
  B["x1"] = singular["x"] ;
  B["x2"] = singular["y"];
  B["x3"] = boundary["x"];
  B["x4"] = boundary["y"];

  vector <expression> tr;
  expression f1 = f.get_component("y0");
  expression f2 = f.get_component("y1");
  tr.push_back(f1.substitute("x1","x").substitute("x2","y") - f1.substitute("x3","x").substitute("x4","y"));
  tr.push_back(f2.substitute("x1","x").substitute("x2","y") - f2.substitute("x3","x").substitute("x4","y"));
  tr.push_back(determina.substitute("x1","x").substitute("x2","y"));

  Map f(tr);
  if (verbose) {f.out();}

  equation(f,B);
  newton n(f);

  if (verbose) 
    {
      B.out();
      f.out();
    }

  domain x = B;
 
  bool finish = false;
  int nb = 0;
  while (finish == false)
    {
      x = n.operator_newton(x);
      if (verbose) {x.out();}
      nb++;
      finish = (nb > 5) || x.subset(B) || x.disjoint(B);
    }
  
  if (x.disjoint(B)) return 2;
  if (x.subset(B)) return 5; 
  return 6;
}




//--------------------------------------------------------------------------
//
//   Méthode qui parcours et découpe la somme directe du 
//   bord et de la singularité (avec les soucis d'intersection dans la source <- pas encore fait)
//
//--------------------------------------------------------------------------
int thom::singular_boundary(domain initial , bool verbose  )
{
  res.singular_boundary.clear();
  int k=0;
  int result = 0;
  vector < pair < domain, domain > > L_courante; // < domain singular , domain boundary > 

  // liste initiale des éléments de la singularité
  domain zero2;
  zero2["y0"] = interval(0);
  sivia courbe = sivia(Singular_curve,zero2);
  courbe.SIVIAfile(initial,0.1);
  vector < domain > L_cur_1 = courbe.L_sivia_MB;

  // liste initiale des éléments du bords
  vector < domain > L_cur_2 = initial.boundary();  

  // création de la liste initiale à partir du bord
  if (verbose) {
    cout << endl << "----------------------------------------------------------------------" << endl ;
    cout << "création de la liste initiale à partir de tout l'espace et du bord " << endl ;
    cout << "-------------------------------------------------------------------" << endl ;
    
  } 

  int taille_1 = L_cur_1.size();    
  int taille_2 = L_cur_2.size();    
  for (int j = 0; j < taille_1; j++)
    {
      domain X1 = L_cur_1[j];
      for ( int i = 0 ; i < taille_2; i++)
	{
	  domain X2 = L_cur_2[i];
	  pair < domain , domain > P = make_pair (X1,X2);		
	  L_courante.push_back(P);
	  if (verbose) { cout << i << ","  << j << endl;   X1.out();    X2.out(); }
	}
    }
  // fin de la création de la liste initiale à partir du bord

  bool not_finish = true;
  while (not_finish == true)
    {

      not_finish = false; 
      vector < int > to_be_cut;
      int taille4 = L_courante.size();  

      if (verbose) 
	{
	  cout << "-------------------------------------" << endl << "Passe n° : " << k ;
	  cout << ", taille de la liste à traiter  : "<<  taille4 << endl;
	}

      for (int j = 0; j < taille4; j++)
	{
	  domain X1 = L_courante[j].first;
	  domain X2 = L_courante[j].second;
	  int etat = singular_boundary(X1,X2, verbose);
	  if (verbose)
	    { cout << "couple n° : " << j <<  ", état : "  << etat <<   endl;}
	  
	  // si cas non favorable alors futur découpage 
	  if (etat!=1 && etat!=2 && etat!=3  && etat!=4 && etat!=5 && etat!=7 )     
	    {
	      not_finish = true;
	      to_be_cut.push_back(j);     
	    }
	  // sinon ajout à la liste 
	  else 
	    { 
	      pair <domain, domain > Py = make_pair(X1,X2);    
	      res.singular_boundary.push_back(make_pair(Py,etat)); 
	      if (etat==5) { res.singular_boundary_v.push_back(Py); }
	      if (etat==7) { res.singular_boundary_v.push_back(make_pair(X1.intersect(X2),X1.intersect(X2))); }
	    }
	}

      if (verbose)
	{ cout << "Fin de la passe " << k  << ", cardinal des couples encore à traiter : " 
	       <<  to_be_cut.size() << endl ;}

      // -----------------------------------------
      // découpage des couples qui posent problème
      // -----------------------------------------
      vector < pair < domain , domain  > > Future_L_courante ; 
      
      taille4 = to_be_cut.size();  
      for (int j = 0; j < taille4; j++)
	{ 
	  int ref_couple = to_be_cut[j] ;
	  domain I = L_courante[ref_couple].first;
	  domain J = L_courante[ref_couple].second;
	  mesh Mi(I,2);
	  mesh Mj(J,2);	      	      
	  
	  for (int mi = 0; mi< Mi.size();mi++)
	    {
	      for (int mj = 0; mj< Mj.size();mj++)
		{
		  pair < domain , domain > P = make_pair ( Mi[mi],Mj[mj]);
		  Future_L_courante.push_back(P);
		}
	    }
	}
       k++;

       L_courante.clear();
       L_courante = Future_L_courante;
       Future_L_courante.clear();
       result = to_be_cut.size();
       to_be_cut.clear();

       if ( k> 10) 
	 {
	   int taille5 = L_courante.size();  
	   for (int j = 0; j < taille5; j++)
	     {
	       res.singular_boundary.push_back(make_pair(L_courante[j],8));
	     }
	   not_finish = false;
          }

    }

  if (verbose)
    {
      cout << "cardinal des solutions :" << res.singular_boundary.size()  << endl;
    };

  return result;
};






// fonction qui remplit la liste singular_singular avec des pairs de pavés (X1,X2)
// où X1 et X2 contiennent chacun une singularité et 
// il existe x1 \in X1 et x2 \in X2 tel que f(x1) = f(x2)
void thom::singular_singular(domain initial , bool verbose) 
{
   domain zero2;
   zero2["y0"] = interval(0);

   // first step : generate a paving containing 
   sivia courbe = sivia(Singular_curve,zero2);
   courbe.SIVIAfile(initial,0.1);


   vector < pair < domain, domain > > L_cur;
   vector < pair < domain, domain > > Future_L_cur;


   for ( int i = 0 ; i < courbe.L_sivia_MB.size(); i++)
	{
      	  domain temp = courbe.L_sivia_MB[i];
	  for ( int j = 0 ; j <= i; j++)
	    {
	      domain temp2 = courbe.L_sivia_MB[j];
	      pair < domain , domain  > Init;
	      Init.first = temp;
	      Init.second = temp2;
	      L_cur.push_back(Init);
	    }
	    }


  
   //   pair < domain , domain  > Init;
   /*   Init.first = initial;
   Init.second = initial;
   L_cur.push_back(Init);*/

   vector < int > to_be_cut;
   int k=0;

   bool not_finish = true; 
   while (not_finish == true)
     {
       not_finish = false;
       vector < int > future_new_list;
       int taille = L_cur.size();  
	       if (verbose)
		 {cout << "_____________ passe :" << k <<" __________taille :"<<  taille << "______________________" << endl;}
       for (int j = 0; j < taille; j++)
	 {
	   domain X1 = L_cur[j].first;
	   domain X2 = L_cur[j].second;
	   int etat = singular_singular(X1,X2);
	   bool egaux = (X1==X2);
	   if (etat!=5 && etat!=2 && etat!=3  && etat!=1 && etat!=4 )     
	     {

	       if (verbose)
		 {
		   cout <<  "couple :" <<  j<< " egaux ?"  <<  egaux  << endl;
		   cout <<  "image disjointe :" << evalue(X1).disjoint(evalue(X2)) << endl;
		   cout <<  "pavé i :"  << endl; 
		   X1.out();
		   cout <<  "pavé j :"  << endl;
		   X2.out();
		   cout << " resultat : "  << etat << endl;
		   cout << "---------------------------------"  << endl; 
		   not_finish = true;
		   to_be_cut.push_back(j);     
		 }
	     }
	   if (etat==5)
	     {
	       res.singular_singular_v.push_back(L_cur[j]);
	     }
	 }
       
       if (verbose) {
	  cout << "Fin de la passe " << k  << ", cardinal des couples encore à traiter : " <<  to_be_cut.size() << endl ;
       };
       
       
       // -----------------------------------------
       // découpage des couples qui posent problème
       // -----------------------------------------
       vector < pair < domain , domain  > > Future_L_courante ; 
       
       int taille4 = to_be_cut.size();  
       for (int j = 0; j < taille4; j++)
	 { 
	   int ref_couple = to_be_cut[j] ;
	   domain I = L_cur[ref_couple].first;
	   domain J = L_cur[ref_couple].second;
	   mesh Mi(I,2);
	   mesh Mj(J,2);	      	      
	   
	   if (I!=J)
	     for (int mi = 0; mi< Mi.size();mi++)
	       {
		 for (int mj = 0; mj< Mj.size();mj++)
		   {
		     pair < domain , domain > P = make_pair ( Mi[mi],Mj[mj]);
		     Future_L_cur.push_back(P);
		   }
	       }
	   
	   if (I==J)
	     for (int mi = 0; mi< Mi.size();mi++)
	       {
		 for (int mj = 0; mj< Mj.size();mj++)
		   {
		     if (mi <= mj)
		       {
			 pair < domain , domain > P = make_pair ( Mi[mi],Mi[mj]);
			 Future_L_cur.push_back(P);
		       }
		   }
	       }
	 }
       k++;
       
       L_cur.clear();
       L_cur = Future_L_cur;
       Future_L_cur.clear();
       to_be_cut.clear();
       
       if ( k> 10 ) 
	 {
	   not_finish = false;
	 }
       
     }
   
   if (verbose)
     {cout << "cardinal des solutions :" << res.singular_singular_v.size();}

};



void thom::improve_singular_singular()
{

  int n = res.singular_singular_v.size();
  for (int i = 0; i<n; i++)
    {
     domain X1 = res.singular_singular_v[i].first;
     domain X2 = res.singular_singular_v[i].second;

     domain X;
     X["x1"] = X1["x"] ;
     X["x2"] = X1["y"];
     X["x3"] = X2["x"];
     X["x4"] = X2["y"];      
     
     X = newton_singular_singular.operator_newton(X);

     X1["x"] = X["x1"];
     X1["y"] = X["x2"];
     X2["x"] = X["x3"];
     X2["y"] = X["x4"];      

     res.singular_singular_v[i] = make_pair(X1,X2);
    }
}




void thom::improve_singular_boundary()
{
  int n = res.singular_boundary_v.size();
  for (int i = 0; i<n; i++)
    {
     domain singular = res.singular_boundary_v[i].first;
     domain boundary = res.singular_boundary_v[i].second;


     if (! singular.disjoint(boundary))
       {

	 Map Singular_curve_temp = Map(determina);
	 domain boundary_copie = boundary;
	 equation(Singular_curve_temp,boundary_copie);
	 
	 newton n(Singular_curve_temp);
       
	 domain x = singular ;
	 
	 x= n.operator_newton(boundary_copie);

	 domain result = boundary;
	 if (boundary["x"].isPoint())
	   {
	     result["x"] = boundary["x"];
	     result["y"] = x["y"];
	   }
	 else
	   {
	     result["y"] = boundary["y"];
	     result["x"] = x["x"];
	   }
	 
	   
	 res.singular_boundary_v[i] = make_pair(result,result);	 
       }



     else 


       {

	 domain B;
	 B["x1"] = singular["x"] ;
	 B["x2"] = singular["y"];
	 B["x3"] = boundary["x"];
	 B["x4"] = boundary["y"];
	 
	 vector <expression> tr;
	 expression f1 = f.get_component("y0");
	 expression f2 = f.get_component("y1");
	 tr.push_back(f1.substitute("x1","x").substitute("x2","y") - f1.substitute("x3","x").substitute("x4","y"));
	 tr.push_back(f2.substitute("x1","x").substitute("x2","y") - f2.substitute("x3","x").substitute("x4","y"));
	 tr.push_back(determina.substitute("x1","x").substitute("x2","y"));
	 
	 Map f(tr);
	 
	 equation(f,B);
	 newton n(f);
	 
	 domain x = B;
	 
	 bool finish = false;
	 x = n.operator_newton(x);
	 x = x.intersect(B);

	 
	 x = n.operator_newton(x);
	 x = x.intersect(B);

	 x = n.operator_newton(x);
	 x = x.intersect(B);


	 x = n.operator_newton(x);
	 x = x.intersect(B);
	

 	 domain result_sing;
	 domain result_bound;


	 if (boundary["x"].isPoint())
	   {
	     result_sing["x"] = x["x1"];
	     result_sing["y"] = x["x2"];
	     result_bound["x"] = boundary["x"];
	     result_bound["y"] = x["x4"];

	   }
	 else
	   {
	     result_sing["x"] = x["x1"];
	     result_sing["y"] = x["x2"];
	     result_bound["x"] = x["x3"];
	     result_bound["y"] = boundary["y"];

	   }


		 res.singular_boundary_v[i] = make_pair(result_sing,result_bound);	 
	 



       }


    }
}



void thom::improve_cusp()
{
  int n = res.cusp_v.size();
  for (int j = 0; j<100; j++)
  for (int i = 0; i<n; i++)
    {
      res.cusp_v[i] =  newton_cusp.operator_newton(res.cusp_v[i]);      
    };
}


void thom::improve_boundary_boundary()
{
  int n = res.boundary_boundary.size();
  for (int j = 0; j<100; j++)
  for (int i = 0; i<n; i++)
    {
      res.boundary_boundary_v[i] =  improve_boundary_boundary(res.boundary_boundary_v[i]);      
    };
  /*
    pour afficher à quelle précision on a les intersections de bords  
    for (int i = 0; i<n; i++)
    {
      cout << res.boundary_boundary[i].first.width() << endl;
    };
  */
}



int thom::Cusp(domain X)
{
    double epsilon;

    int k=0;

    list <domain> L_cur;
    L_cur.clear();

    // on place P dans la list L
    L_cur.push_back(X);
    
    domain zero;
    zero["y0"] = interval(0);
    zero["y1"] = interval(0);

    int count_les_jaunes = 0;
    while (!L_cur.empty())
      {
	domain P;
	P=L_cur.front();
	L_cur.pop_front();
	
	domain x;
	x = newton_cusp.operator_newton(P);

	if ( x.subset(P) && ! zero.subset(xi.evalue(P)) && (one_to_one_cusp(x,false)) )
	  {
	    res.cusp.push_back(make_pair(P,5));
	    res.cusp_v.push_back(P);
	  }
	else if ( ! zero.subset(cusp.evalue(P)) )
	  {
	    res.cusp.push_back(make_pair(P,4));
	  }
	else if ( x.disjoint(P) )
	  {
	    res.cusp.push_back(make_pair(P,3));
	  }
	else
	  {
	    mesh M(P,2);
	    for (mesh::const_iterator i = M.begin(); i!=M.end();++i)
	      L_cur.push_back(*i);
	  } 
      }
    return L_cur.size();
}




bool thom::one_to_one_cusp(domain x , bool verbose)
{
  domain gradient_p0;
  gradient_p0["x"] = gradient.coefficient["y0"]["x"].Evalue(x);
  gradient_p0["y"] = gradient.coefficient["y0"]["y"].Evalue(x);

  domain imdfp0;
  imdfp0 = df.evalue(x).image(gradient_p0);


  interval a = interval(-1) *  imdfp0["y1"]   ;
  interval b = imdfp0["y0"]   ;
  

  vector_field xi2; 
  xi2.expr2["x"] = xi.get_component("y0");
  xi2.expr2["y"] = xi.get_component("y1");
  //  xi2.out();

  //  xi2.Taylor(zero,3).out();
  taylor third_derivative = f.derive(xi2,3,x);
  
  interval pb = a*third_derivative["y0"][3] + b*third_derivative["y1"][3];

  if (verbose) 
    { gradient_p0.out();
      imdfp0.out();
      third_derivative.out();
      cout << pb << endl;
    }
   return (!interval(0).subset(pb));; 
}
